#
# Copyright 2012 Mark McCracken
#
# This file is part of PeriodicDocumentScan, a set of simple scripts to 
# facilitate the scanning of periodic documents (bills, paystubs, statements) to PDF.
#
# PeriodicDocumentScan is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PeriodicDocumentScan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PeriodicDocumentScan.  If not, see <http://www.gnu.org/licenses/>.

package ScanCommon;
use strict;
use base "Exporter";
use Date::Manip::Date;
use File::Basename;
use File::Path qw(mkpath);

my $ourVerbose = 0;
my $ourQualityString = "Lineart";
my $defaultLineartDPI = 150;
my $defaultPhotoDPI = 300;
my $ourDPI = $defaultLineartDPI;
my $ourOutputDirectory = "";
my $ourNumSides = 2;
my $ourRotation = 0;
my $ourBaseTitle = "";
my $ourCurrentDate = new Date::Manip::Date;
my $ourScriptName = "ScanCommon";

sub setScriptName
{
    $ourScriptName = basename(shift);
}

sub usagePreamble
{
    printf("Usage: %s %s [-q qual] [-r deg] [-s n] [-o folder] title\n", 
           $ourScriptName, shift);
}

sub usageBasicParams
{
    print "     -q  Specify quality as one of:\n";
    print "         lineart = scan as $defaultLineartDPI dpi text documents [default]\n";
    print "         photo = scan as $defaultPhotoDPI dpi color photos\n";
    print "     -r  Specify degrees to rotate the scanned image [0]\n";
    print "     -s  Specify how many sides to scan [2]\n";
    print "     -o  Specify a folder to save documents into [.]\n";
    print "  title  Specify base title for all documents [title_date]\n";
}

sub usageCommandTransition
{
    print "\n";
    print "In-session commands:\n";
}

sub usageBasicCommands
{
    print " skip n  Skips \"n\" periods forward and initiates scan\n";
    print "   help  Displays this text, does not initiate scan\n";
    print "   quit  Exits immediately\n";
    print "<enter>  Initiates next scan\n\n";
}

sub currentYMD
{
    if (scalar @_)
    {
        my ($Yr, $Mo, $Dy) = @_;
        if (!$Mo) {$Mo = 1;}
        if (!$Dy) {$Dy = 1;}
        $ourCurrentDate->parse(sprintf("%04d-%02d-%02d", $Yr, $Mo, $Dy));
    }
    
    return $ourCurrentDate->printf("%Y%m%d");
}

sub verbose
{
    if (scalar @_) {$ourVerbose = shift;} 
    
    return $ourVerbose;
}
    
sub maybePrint
{
    if ($ourVerbose) {print @_;}
}

sub quality
{
    if (scalar @_)
    {
        $ourQualityString = lc shift;

        # validate quality, and log it
        my $firstChar = substr($ourQualityString, 0, 1);
        if ($firstChar eq "l")
        {
            $ourQualityString = "Lineart";
            $ourDPI = $defaultLineartDPI;
        }
        elsif ($firstChar eq "p")
        {
            $ourQualityString = "Color";
            $ourDPI = $defaultPhotoDPI;
        }
        else {return 0;}
    }

    return $ourQualityString;
}

sub outputDir
{
    if (scalar @_)
    {
        $ourOutputDirectory = shift;

        # create the output directory if necessary, and log it
        if ($ourOutputDirectory ne "")
        {
            my $numCreated = mkpath($ourOutputDirectory);
            
            unless (-d $ourOutputDirectory)
            {
                print "Failed to create the output directory \"$ourOutputDirectory\".\n";
                exit 1;
            }
            
            if ($numCreated > 0) {maybePrint "Created output directory \"$ourOutputDirectory\".\n";}
            chdir $ourOutputDirectory;
        }
    }
    
    return $ourOutputDirectory;
}

sub numSides
{
    if (scalar @_) {$ourNumSides = shift;}
    return $ourNumSides;
}

sub rotation
{
    if (scalar @_) {$ourRotation = shift;}
    return $ourRotation;
}

sub baseTitle
{
    if (scalar @_)
    {
        $ourBaseTitle = shift;
        
        if ($ourBaseTitle eq "") {return 0;}
        $ourBaseTitle =~ s/\s/_/;
    }

    return $ourBaseTitle;
}

sub logSettings
{
    maybePrint "Scanning in $ourQualityString mode at $ourDPI dpi.\n";
    if ($ourRotation != 0) {maybePrint "Rotating image $ourRotation degrees.\n";}
    maybePrint "Scanning $ourNumSides side(s).\n";
    maybePrint "Using \"$ourBaseTitle\" for a base title.\n";
}

sub getNowPdfMarks()
{
    my @now = localtime;
    return sprintf("%04d%02d%02d%02d%02d%02d", 1900 + $now[5], 1 + $now[4], 
                   $now[3], $now[2], $now[1], $now[0]);
}

sub getCurPdfMarks()
{
    return $ourCurrentDate->printf("%Y%m%d%H%M%S");
}

sub scan
{
    my $outBase = sprintf("%s_%s", $ourBaseTitle, shift);
    maybePrint "Scanning to $outBase" . ".pdf...\n";

    my $scanadfOut = $outBase . "_%d.pnm";
    if ($ourRotation != 0) {$scanadfOut = $outBase . "_%d.x.pnm";}
    my $sourceStr = "ADF Duplex";
    if ($ourNumSides == 1) {$sourceStr = "ADF Front";}
    
    my $scanadfCmd = sprintf("scanadf -o \"%s\" --source \"%s\" --mode %s --resolution %d",
                             $scanadfOut, $sourceStr, $ourQualityString, $ourDPI);
    maybePrint "$scanadfCmd\n";
    system("$scanadfCmd") == 0 or die "scanadf returned $?";
    
    if ($ourRotation != 0)
    {
        open(PREFILES, sprintf("ls %s_*.x.pnm |", $outBase));
        my @preFiles = <PREFILES>;
        close PREFILES;
        
        foreach my $curPre (@preFiles)
        {
            chomp $curPre;
            my $curOut = $curPre;
            $curOut =~ s/\.x\./\./;
            my $unpaperCmd = sprintf("unpaper --pre-rotate %d --no-processing 1 %s %s",
                                     $ourRotation, $curPre, $curOut);
            maybePrint "$unpaperCmd\n";
            system("$unpaperCmd\n") == 0 or die "unpaper on $curPre returned $?";
            unlink $curPre;
        }
    }

    open(PNMFILES, sprintf("ls %s_*.pnm |", $outBase));
    my @pnm_files = <PNMFILES>;
    close PNMFILES;

    foreach my $curpnm (@pnm_files)
    {
        chomp $curpnm;
        my $pntopsCmd = "pnmtops -noturn -rle $curpnm > \$(basename $curpnm .pnm).eps";
        maybePrint "$pntopsCmd\n";
        if (0 == $ourVerbose) {$pntopsCmd = $pntopsCmd . " 2> /dev/null";}
        system("$pntopsCmd") == 0 or die "pnmtops on $curpnm returned $?";
        unlink $curpnm;
    }
    
    my $finalOut = $outBase . ".pdf";
    my $psout = sprintf("%s_*.eps", $outBase);

    open(PDFMARKS, ">" . $outBase . "_pdfmarks");
    printf(PDFMARKS "[ /Title (%s)\n  /ModDate (D:%s)\n", $outBase, getNowPdfMarks());
    printf(PDFMARKS "  /CreationDate (D:%s)\n", getCurPdfMarks());
    print PDFMARKS "  /DOCINFO pdfmark\n";
    close PDFMARKS;
    
    my $gsCmd = sprintf("gs -q -dSAFER -dNOPAUSE -dBATCH -sOutputFile=%s -sDEVICE=pdfwrite %s %s_pdfmarks",
                        $finalOut, $psout, $outBase);
                        
    maybePrint "$gsCmd\n";
    system("$gsCmd") == 0 or die "gs $psout to $finalOut returned $?";
    system("rm $psout");    # used system(rm) here because of wildcarded filename
    unlink $outBase . "_pdfmarks";
}

sub processBasicCmd
{
    my $input = lc <STDIN>;
    chomp $input;
    if (length($input) == 0) {return ("next", "one");}
    my $firstChar = substr($input, 0, 1);
    
    if ($firstChar eq "q")
    {
        print "Bye!\n";
        exit 0;
    }
    elsif ($firstChar eq "h")
    {
        return ("help", "usage");
    }
    elsif ($firstChar eq "s")
    {
        my @tokens = split(/\s/, $input);
        my $numTokens = @tokens;
        
        if ($numTokens != 2)
        {
            print "Syntax: \"skip n\", where n is the number of periods to skip.\n";
            print "Example: \"skip 4\" would skip forward 4 periods.\n";
            return ("help", "usage");
        }
        else
        {
            my $numPeriods = $tokens[-1];
            if ($numPeriods < 1) {$numPeriods = 1;}

            ScanCommon::maybePrint "Skipping forward $numPeriods periods...\n";
            return ("skip", $numPeriods);
        }
    }
    else
    {
        return split(/\s/, $input);
    }
}


















1;
