#!/usr/bin/perl
#
# Copyright 2012 Mark McCracken
#
# This file is part of PeriodicDocumentScan, a set of simple scripts to 
# facilitate the scanning of periodic documents (bills, paystubs, statements) to PDF.
#
# PeriodicDocumentScan is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PeriodicDocumentScan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PeriodicDocumentScan.  If not, see <http://www.gnu.org/licenses/>.

use strict;
use ScanCommon;
use Date::Manip::Date;
use Date::Manip::Delta;

ScanCommon::setScriptName $0;
my $ourMonthStyle = "numeric";
my $ourCurrentDate = new Date::Manip::Date;

sub getDateString
{
    return $ourCurrentDate->printf("\%Y-" . $ourMonthStyle . "-\%d");
}

sub setDefaultDateVars
{
    $ourCurrentDate->set("DATE", localtime(time));
}

sub commandUsage
{
    ScanCommon::usageCommandTransition;
    print " date d  Sets date to \"d\" and initiates scan\n";
    ScanCommon::usageBasicCommands;
}

sub usage
{
    ScanCommon::usagePreamble "[-m style] [-d startDate] ";
    print "     -m  Specify month tag style as one of:\n";
    print "         numeric: (01-12) [default]\n";
    print "         short:   (Jan-Dec)\n";
    print "         long:    (January-December)\n";
    print "     -d  Specify starting date in YYYY-MM-DD format [today]\n";
    ScanCommon::usageBasicParams;
    commandUsage;
}

sub dieUsage
{
    usage;
    print join("\n", @_), "\n";
    exit 1;
}

sub parseCommandLine
{
    use Getopt::Std;
    use Switch;
    use File::Path;
    
    if ($#ARGV == -1) {usage; exit 0;}
    
    my %opts;
    getopt('mdqosr ', \%opts);
    
    foreach my $key (keys(%opts))
    {
        switch ($key)
        {
        case 'm' {$ourMonthStyle = lc $opts{$key};}
        case 'd' {$ourCurrentDate->parse($opts{$key});}
        case 'q' {ScanCommon::quality($opts{$key}) || 
                  dieUsage("Invalid quality string \"$opts{$key}\"",
                           "Valid values: \"lineart\", \"photo\"");}
        case 'o' {ScanCommon::outputDir $opts{$key};}
        case 's' {ScanCommon::numSides $opts{$key};}
        case 'r' {ScanCommon::rotation $opts{$key};}
        case 'h' {usage; exit 0;}
        case 'v' {ScanCommon::verbose 1;}
        }
    }
    
    ScanCommon::baseTitle(@ARGV[-1]) or dieUsage("Missing a base title");
    
    # validate month style
    my $firstChar = substr($ourMonthStyle, 0, 1);
    if ($firstChar eq "n") {$ourMonthStyle = "\%m";}
    elsif ($firstChar eq "s") {$ourMonthStyle = "\%b";}
    elsif ($firstChar eq "l") {$ourMonthStyle = "\%B";}
    else
    {
        dieUsage("Invalid month tag style \"$ourMonthStyle\".",
                 "Valid values: \"numeric\", \"short\", \"long\".");
    }

    ScanCommon::logSettings;
}

sub addPeriods
{
    my $numPrds = shift;
    $numPrds *= 2;

    my $delta = $ourCurrentDate->new_delta;
    $delta->parse("0:0:$numPrds:0:0:0:0");
    $ourCurrentDate = $ourCurrentDate->calc($delta);
    my $dateStr = $ourCurrentDate->printf("%Y-%m-%d");
    ScanCommon::maybePrint "New current date is $dateStr\n";
}

sub getYMD
{
    return ($ourCurrentDate->printf("%Y"), $ourCurrentDate->printf("%m"), $ourCurrentDate->printf("%d"));
}

sub scan
{
    ScanCommon::currentYMD(getYMD());
    ScanCommon::scan(getDateString());
    addPeriods 1;
}

setDefaultDateVars();
parseCommandLine();

printf("Load source for %s_%s and press enter: ", ScanCommon::baseTitle, getDateString());

while (1)
{
    my @todo = ScanCommon::processBasicCmd;
    if ($todo[0] eq "help") {commandUsage;}
    elsif ($todo[0] eq "skip") {addPeriods $todo[1]; scan;}
    elsif ($todo[0] eq "date") {$ourCurrentDate->parse($todo[1]); scan;}
    elsif ($todo[0] eq "next") {scan;}
    else
    {
        print "Invalid command \"@todo\".\n";
        print "Type \"help\" for assistance, or \"quit\" to quit.\n";
    }

    printf("Load source for %s_%s and press enter: ", ScanCommon::baseTitle, getDateString());
}

