#!/usr/bin/perl

#
# Copyright 2012 Mark McCracken
#
# This file is part of PeriodicDocumentScan, a set of simple scripts to 
# facilitate the scanning of periodic documents (bills, paystubs, statements) to PDF.
#
# PeriodicDocumentScan is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PeriodicDocumentScan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PeriodicDocumentScan.  If not, see <http://www.gnu.org/licenses/>.

use strict;
use ScanCommon;

ScanCommon::setScriptName $0;

our $ourMonthStyle = "short";
our $ourCurrentMonth = 99;
our $ourCurrentMonthString = "no idea!";
our $ourIncludeYear = 0;
our $ourCurrentYear = 9999;
our $ourIncludeDay = 0;
our $ourCurrentDay = 1;

sub getMonthString
{
    my @longMonthStrings = ("", "January", "February", "March", "April", "May", "June",
                            "July", "August", "September", "October", "November", "December");
    my @shortMonthStrings = ("", "Jan", "Feb", "Mar", "Apr", "May", "Jun",
                             "Jul", "Aug", "Sep", "Oct", "Nov", "Dec");
    
    if ($ourMonthStyle eq "short") {return @shortMonthStrings[$ourCurrentMonth];}
    elsif ($ourMonthStyle eq "long") {return @longMonthStrings[$ourCurrentMonth];}
    else {return sprintf("%02d", $ourCurrentMonth);}
}

sub setDefaultDateVars
{
    my @timeData = localtime(time);
    $ourCurrentMonth = @timeData[4] + 1;
    $ourCurrentMonthString = getMonthString();
    $ourCurrentYear = @timeData[5] + 1900;
}

sub commandUsage
{
    ScanCommon::usageCommandTransition;
    ScanCommon::usageBasicCommands;
}

sub usage
{
    ScanCommon::usagePreamble "[-m style] [-f first] [-y year] [-d day] ";
    print "     -m  Specify month tag style as one of:\n";
    print "         numeric: (01-12)\n";
    print "         short:   (Jan-Dec) [default]\n";
    print "         long:    (January-December)\n";
    print "     -f  Specify first month [$ourCurrentMonth, aka $ourCurrentMonthString]\n";
    print "     -y  Set starting year and include in filename in form YYYY-MM [MM only]\n";
    print "     -d  Include the specified day num in the date [YYYY-]MM-DD\n";
    ScanCommon::usageBasicParams;
    commandUsage;
}

sub dieUsage
{
    usage;
    print join("\n", @_), "\n";
    exit 1;
}

sub setCurrentMonth
{
    my $monthNum = shift;
    if ($monthNum < 1 || $monthNum > 12)
    {
        dieUsage "Invalid starting month $monthNum. Must be [1..12].";
    }
    
    my $oldMonth = $ourCurrentMonth;
    $ourCurrentMonth = $monthNum;
    $ourCurrentMonthString = getMonthString();

    if ($monthNum != $oldMonth)
    {
        ScanCommon::maybePrint "New current month: $ourCurrentMonthString\n";
    }
}

sub parseCommandLine
{
    use Getopt::Std;
    use Switch;
    use File::Path;
    
    my %opts;
    getopt('mfydqosr ', \%opts);

    if ($#ARGV == -1) {usage; exit 0;}
    
    foreach my $key (keys(%opts))
    {
        switch ($key)
        {
        case 'm' {$ourMonthStyle = lc $opts{$key};}
        case 'f' {setCurrentMonth($opts{$key});}
        case 'y' {$ourCurrentYear = $opts{$key}; $ourIncludeYear = 1;}
        case 'd' {$ourCurrentDay = $opts{$key}; $ourIncludeDay = 1;}
        case 'q' {ScanCommon::quality($opts{$key}) || 
                  dieUsage("Invalid quality string \"$opts{$key}\"",
                           "Valid values: \"lineart\", \"photo\"");}
        case 'o' {ScanCommon::outputDir $opts{$key};}
        case 's' {ScanCommon::numSides $opts{$key};}
        case 'r' {ScanCommon::rotation $opts{$key};}
        case 'h' {usage; exit 0;}
        case 'v' {ScanCommon::verbose 1;}
        }
    }
    
    ScanCommon::baseTitle(@ARGV[-1]) or dieUsage("Missing a base title");
    
    # validate month style
    my $firstChar = substr($ourMonthStyle, 0, 1);
    if ($firstChar eq "n") {$ourMonthStyle = "numeric";}
    elsif ($firstChar eq "s") {$ourMonthStyle = "short";}
    elsif ($firstChar eq "l") {$ourMonthStyle = "long";}
    else
    {
        dieUsage("Invalid month tag style \"$ourMonthStyle\".",
                 "Valid values: \"numeric\", \"short\", \"long\".");
    }
    
    ScanCommon::maybePrint "Using $ourMonthStyle months.\n";

    # make sure we have the right month string    
    setCurrentMonth($ourCurrentMonth);
    
    # log year setting
    if ($ourIncludeYear == 1) 
    {
        ScanCommon::maybePrint "Including the year, starting with $ourCurrentYear.\n";
    }
    else 
    {
        ScanCommon::maybePrint "Not including the year.\n";
    }
    
    # log day setting
    if ($ourIncludeDay == 1) 
    {
        ScanCommon::maybePrint "Including a day-tag of $ourCurrentDay.\n";
    }

    ScanCommon::logSettings;    
}

sub addMonths
{
    my $num = shift;
    $num += $ourCurrentMonth;
    
    while ($num > 12)
    {
        $num -= 12;
        $ourCurrentYear += 1;
    }
    
    setCurrentMonth($num);
}

sub buildFilename
{
    my $dateStr = $ourCurrentMonthString;
    
    if ($ourIncludeYear == 1) {$dateStr = sprintf("%04d-%s", $ourCurrentYear, $dateStr);}
    if ($ourIncludeDay == 1) {$dateStr = sprintf("%s-%02d", $dateStr, $ourCurrentDay);}
    
    return $dateStr;
}

sub scan
{
    ScanCommon::currentYMD($ourCurrentYear, $ourCurrentMonth, $ourCurrentDay);
    ScanCommon::scan(buildFilename());
    addMonths 1;
}

setDefaultDateVars();
parseCommandLine();

printf("Load source for %s_%s and press enter: ", ScanCommon::baseTitle, buildFilename());

while (1)
{
    my @todo = ScanCommon::processBasicCmd;
    if ($todo[0] eq "help") {commandUsage;}
    elsif ($todo[0] eq "skip") {addMonths $todo[1]; scan;}
    elsif ($todo[0] eq "next") {scan;}
    else
    {
        print "Invalid command \"@todo\".\n";
        print "Type \"help\" for assistance, or \"quit\" to quit.\n";
    }

    printf("Load source for %s_%s and press enter: ", ScanCommon::baseTitle, buildFilename());
}

