#!/usr/bin/perl

#
# Copyright 2012 Mark McCracken
#
# This file is part of PeriodicDocumentScan, a set of simple scripts to 
# facilitate the scanning of periodic documents (bills, paystubs, statements) to PDF.
#
# PeriodicDocumentScan is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PeriodicDocumentScan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PeriodicDocumentScan.  If not, see <http://www.gnu.org/licenses/>.

use strict;
use ScanCommon;

our $ourCurrentYear = 9999;
ScanCommon::setScriptName $0;

sub setDefaultDateVars
{
    my @timeData = localtime(time);
    $ourCurrentYear = @timeData[5] + 1900;
}

sub commandUsage
{
    ScanCommon::usageCommandTransition;
    print " year n  Sets year to \"n\" and initiates scan\n";
    ScanCommon::usageBasicCommands;
}

sub usage
{
    ScanCommon::usagePreamble "[-y year] ";
    print "     -y  Specify starting year [$ourCurrentYear]\n";
    ScanCommon::usageBasicParams;
}

sub dieUsage
{
    usage;
    print join("\n", @_), "\n";
    exit 1;
}

sub parseCommandLine
{
    use Getopt::Std;
    use Switch;
    use File::Path;
    
    if ($#ARGV == -1) {usage; exit 0;}

    my %opts;
    getopt('yqosr ', \%opts);
    
    foreach my $key (keys(%opts))
    {
        switch ($key)
        {
        case 'y' {$ourCurrentYear = $opts{$key};}
        case 'q' {ScanCommon::quality($opts{$key}) || 
                  dieUsage("Invalid quality string \"$opts{$key}\"",
                           "Valid values: \"lineart\", \"photo\"");}
        case 'o' {ScanCommon::outputDir $opts{$key};}
        case 's' {ScanCommon::numSides $opts{$key};}
        case 'r' {ScanCommon::rotation $opts{$key};}
        case 'h' {usage; exit 0;}
        case 'v' {ScanCommon::verbose 1;}
        }
    }
    
    ScanCommon::baseTitle(@ARGV[-1]) or dieUsage("Missing a base title!");
    
    # log year setting
    ScanCommon::maybePrint "Starting with $ourCurrentYear.\n";
    ScanCommon::logSettings;
}

sub addYears
{
    my $num = shift;
    $ourCurrentYear += $num;
}

sub scan
{
    ScanCommon::currentYMD($ourCurrentYear, 12, 31);
    ScanCommon::scan(sprintf("%04d", $ourCurrentYear));
    addYears 1;
}

setDefaultDateVars();
parseCommandLine();

printf("Load source for %s_%04d and press enter: ", ScanCommon::baseTitle(), $ourCurrentYear);

while (1)
{
    my @todo = ScanCommon::processBasicCmd;
    if ($todo[0] eq "help") {commandUsage;}
    elsif ($todo[0] eq "skip") {addYears $todo[1]; scan;}
    elsif ($todo[0] eq "next") {scan;}
    elsif (substr($todo[0], 0, 1) eq "y")
    {
        my $numTokens = @todo;
        if ($numTokens != 2)
        {
            print "Syntax: \"year n\", where n is the year to jump to.\n";
            print "Example: \"year 2004\" would start scanning 2004.\n";
        }
        else
        {
            $ourCurrentYear = $todo[-1];

            ScanCommon::maybePrint "Jumping to $ourCurrentYear...\n";
            scan;
        }
    }
    else
    {
        print "Invalid command \"@todo\".\n";
        print "Type \"help\" for assistance, or \"quit\" to quit.\n";
    }

    printf("Load source for %s_%04d and press enter: ", ScanCommon::baseTitle(), $ourCurrentYear);
}

